package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.dto.PayPageDto;
import cn.wisenergy.service.app.PayService;

import cn.wisenergy.service.util.SignDemo;
import cn.wisenergy.service.util.SignUtil;
import cn.wisenergy.service.wxpay.WxCommon;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import okhttp3.HttpUrl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;
import java.security.spec.InvalidKeySpecException;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;

/**
 * @author 86187
 */
@Service
@Slf4j
public class PayServiceImpl implements PayService {

    @Autowired
    private RestTemplate restTemplate;

    @Override
    public R<String> wxPay(PayPageDto payPageDto) throws UnsupportedEncodingException, NoSuchAlgorithmException, SignatureException, InvalidKeySpecException, InvalidKeyException {
        if (null == payPageDto || null == payPageDto.getTotal()) {
            return R.error("入参不能为空!");
        }

        final HttpHeaders headers = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
        headers.setContentType(type);

        long timestamp = System.currentTimeMillis() / 1000;
        String nonceStr = UUID.randomUUID().toString().replace("-", "");
        String url = "v3/pay/transactions/native";
        String method = "POST";
        String tradeNo = "21" + System.currentTimeMillis();
        HttpUrl httpurl = HttpUrl.parse(WxCommon.WX_PAY_URL);

        //构造签名参数
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("appid", WxCommon.APP_ID);
        jsonObject.put("mchid", WxCommon.MCHID);
        jsonObject.put("timestamp", timestamp);
        jsonObject.put("nonce_str", nonceStr);
        jsonObject.put("url", url);
        jsonObject.put("method", method);
        jsonObject.put("description", "充值");
        jsonObject.put("out_trade_no", tradeNo);
        jsonObject.put("notify_url", WxCommon.NOTIFY_URL);
        jsonObject.put("amount", payPageDto);

        String sign = SignDemo.getToken(method, httpurl, jsonObject.toJSONString(),nonceStr,timestamp);
        headers.add("Wechatpay-Signature", sign);

        HttpEntity<JSONObject> formEntity = new HttpEntity<>(jsonObject, headers);
        ResponseEntity<JSONObject> responseEntity = this.restTemplate.postForEntity(WxCommon.WX_PAY_URL, formEntity, JSONObject.class);

        return R.ok(Objects.requireNonNull(responseEntity.getBody()).toJSONString());
    }
}
